#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include "Maths.h"
#include "math/Vec2f.h"

#define SCREEN_WIDTH    640
#define SCREEN_HEIGHT   480

int maxLevel = 0;

void vonKoch(Vec2f p1, Vec2f p2, int level)
{
    if (level >= maxLevel)
    {
        gfx.line(p1.x, p1.y, p2.x, p2.y, Color(255, 255, 255));
        return;
    }

    Vec2f   n[8];

    Vec2f   v12  = (p2 - p1) * 4.0f / 15.0f;
    Vec2f   v12b = v12;
    Vec2f   v12c = v12;

    v12b.rotate(DEG_TO_RAD(-80));
    v12c.rotate(DEG_TO_RAD(80));

    n[0] = p1 + v12;
    n[7] = p2 - v12;

    n[1] = n[0] + v12b;
    n[6] = n[7] - v12b;

    n[2] = n[1] + v12;
    n[5] = n[6] - v12;

    n[3] = n[2] + v12c;
    n[4] = n[5] - v12c;

    vonKoch(  p1, n[0], level + 1);
    vonKoch(n[0], n[1], level + 1);
    vonKoch(n[1], n[2], level + 1);
    vonKoch(n[2], n[3], level + 1);
    vonKoch(n[3], n[4], level + 1);
    vonKoch(n[4], n[5], level + 1);
    vonKoch(n[5], n[6], level + 1);
    vonKoch(n[6], n[7], level + 1);
    vonKoch(n[7],   p2, level + 1);
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Von Koch 11", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();

    Vec2f   left (SCREEN_WIDTH/2 - SCREEN_WIDTH / 2, SCREEN_HEIGHT/2);
    Vec2f   right(SCREEN_WIDTH/2 + SCREEN_WIDTH / 2, SCREEN_HEIGHT/2);

    int cnt = 0;

    // wait until we quit
    while (sys.isQuitRequested() == false)
    {
        if (cnt == 0)
        {
            gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));
            vonKoch(left, right, 0);
        }

        cnt++;

        if (maxLevel < 4 && cnt == 50)
        {
            cnt = 0;
            maxLevel++;
        }

        gfx.render();
        sys.wait(20);
        sys.processEvents();
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
